/****************************************************************** 
 * @author Alois Kirchsteiger member of the isac team
 * Copyright (c) 2004 by Alois Kirchsteiger
 * create 10.12.2004, 12:51:27
 * Insitute for Softwaretechnology, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui;

import isac.bridge.BridgeRMI;
import isac.interfaces.IToGUI;
import isac.interfaces.IToUser;
import isac.useractions.IUIAction;
import isac.useractions.IUserAction;
import isac.util.CalcChanged;
import isac.util.CalcEvent;
import isac.util.WindowApplicationPaths;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.server.UnicastRemoteObject;

import org.apache.log4j.Logger;

/**
 * @author Alois Kirchsteiger 10.12.2004 12:51:27
 * 
 * This class has quickly been introduced in order to actually pass the
 * CalcChanged through to the Worksheet; WN050418 redesign RMI ?
 *  
 */
public class WorksheetRMI extends UnicastRemoteObject implements IToUser,
		IToGUI {

	Worksheet ws_;

	private static final Logger logger = Logger.getLogger(WorksheetRMI.class.getName());

	/**
	 * @throws RemoteException
	 */
	protected WorksheetRMI(Worksheet ws) throws RemoteException {
		super();
		rmiBind();
		ws_ = ws;
	}

	/**
	 * The Worksheet only gets CalcChanged, while CalcMessage
     * is sent via IUIAction.
	 * 
	 * @see isac.interfaces.IToUser#calcResponse(isac.util.CalcEvent)
	 */
	public void calcResponse(CalcEvent event) throws RemoteException {
	    if(logger.isInfoEnabled())
	    {
		logger.info(" WS<-. . : informListeners (unc="
				+ ((CalcChanged) event).getLastUnchangedFormula().toSMLString() + ", del="
				+ ((CalcChanged) event).getLastDeletedFormula().toSMLString() + ", gen="
				+ ((CalcChanged) event).getLastGeneratedFormula().toSMLString());
		logger.info(" WS<-. . : getCalcChangedEvent(" + ")");
	    }
		ws_.calcChanged((CalcChanged) event);
	}

	private void rmiBind() {
		if (System.getSecurityManager() == null) {
			System.setSecurityManager(new RMISecurityManager());
		}

//		try {
//			LocateRegistry.createRegistry(1066);
//		} catch (java.rmi.RemoteException exc2) {
//			System.err.println("can not create registry: " + exc2.getMessage());
//		}

		String name = WindowApplicationPaths.WORKSHEET_RMI;

		try {
			System.out.println("try to bind as " + name);
			Naming.rebind(name, this);
			System.out.println("WorksheetRMI bound to " + name);
			//WN040906 was Object Manager ...
		} catch (java.rmi.ConnectException e) {
			System.err.println("failed to contact as " + name
					+ " (creating RMI-Server on localhost: 1099)");
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see isac.interfaces.IToGUI#doUIAction(isac.wsdialog.IUserAction)
	 */
	public boolean doUIAction(IUserAction action) throws RemoteException {
		return ws_.doUIAction(action);
	}

    /* (non-Javadoc)
     * @see isac.interfaces.IToGUI#activateUIObject(int, boolean)
     */
    public void activateUIObject(int id, boolean active) {
        // TODO Auto-generated method stub
    }


    public void addUIElement(IUIAction action) throws RemoteException {
        ws_.addUIElement(action);
        
    }

    /* (non-Javadoc)
     * @see isac.interfaces.IToGUI#removeUIElement(isac.wsdialog.IUIAction)
     */
    public boolean removeUIElement(IUIAction action) throws RemoteException {
        return ws_.removeUIElement(action);
    }

    /* (non-Javadoc)
     * @see isac.interfaces.IToGUI#doUIAction(isac.wsdialog.IUIAction)
     */
    public void doUIAction(IUIAction action) throws RemoteException {
        ws_.doUIAction(action);
    }

	public boolean isUIRunning() throws RemoteException {
		return true;
	}
}