/****************************************************************** 
 * @author Alan Krempler member of the isac team
 * Copyright (c) 2004 by Alois Kirchsteiger
 * create 21.09.2004, 10:43:41
 * Insitute for Softwaretechnology, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.users;

import java.io.Serializable;
import java.util.Date;

/**
 * SuccessInfo stores the Information about how successful a user was with a
 * specific CalcElement the SuccessInfo is stored in the <code>UserModel</code>
 * which exists for each <code>User</code> once.
 * 
 * WN070727 preliminary design decision: there is <bf>one</bf> SuccessInfo for
 * <bf>one</bf> example. This results in mean number of SuccessInfo's (between
 * <bf>one</bf> per <code>User</code> and <bf>one</bf> per step; <bf>one</bf>
 * per 'class of steps' seems promising, as soon 'class of steps'[eg. applying
 * the chain-rule] is clarified).
 * 
 * @author Alan Krempler 21.09.2004 10:43:41
 */
public class SuccessInfo implements Serializable {

    static final long serialVersionUID = 7716468065408745731L;

    private int nr_of_tries_;

    private int nr_of_success_;

    private int nr_of_successes_after_last_failure;

    /*
     * as the can be different levels of failure, for each try there will be
     * inserted a number between 0(total failure) and 1(success). the
     * successIndicator stores the average number for all tries, and so its
     * value can also only be betwenn 0 and 1
     */
    private float success_indicator_;

    private Date last_try_;

    public SuccessInfo(float success) {
        nr_of_tries_ = 1;
        // if successful
        if (success == 1) {
            nr_of_success_ = 1;
            nr_of_successes_after_last_failure = 1;
        } else {
            nr_of_success_ = 0;
            nr_of_successes_after_last_failure = 0;
        }
        success_indicator_ = success;
        last_try_ = new Date(java.lang.System.currentTimeMillis());
    }

    public void addSuccess(float success) {
        nr_of_tries_++;
        // if successful
        if (success == 1) {
            nr_of_success_++;
            nr_of_successes_after_last_failure++;
        } else {
            nr_of_successes_after_last_failure = 0;
        }
        success_indicator_ = (success_indicator_ * (nr_of_tries_ - 1) + success)
                / nr_of_tries_;
        last_try_ = new Date(java.lang.System.currentTimeMillis());
    }

    public int getNrOfTries() {
        return nr_of_tries_;
    }

    public int getNrOfSuccess() {
        return nr_of_success_;
    }

    public int getNrOfSuccessesAfterLastFailure() {
        return nr_of_successes_after_last_failure;
    }

    /**
     * as there can be different levels of failure, for each try there will be
     * inserted a number between 0(total failure) and 1(success). the
     * successIndicator stores the average number for all tries, and so its
     * value can also only be betwenn 0 and 1
     */
    public float getSuccessIndicator() {
        return success_indicator_;
    }

    public Date getLastTry() {
        return last_try_;
    }
}